"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.storeEvents = exports.readEntityLatestSnapshot = exports.readEntityEventsSince = exports.rawEventsToEnvelopes = void 0;
const framework_types_1 = require("@boostercloud/framework-types");
const constants_1 = require("../constants");
const keys_helper_1 = require("./keys-helper");
const dynamodb_1 = require("aws-sdk/clients/dynamodb");
const framework_common_helpers_1 = require("@boostercloud/framework-common-helpers");
// eslint-disable-next-line @typescript-eslint/no-magic-numbers
const originOfTime = new Date(0).toISOString();
function rawEventsToEnvelopes(rawEvents) {
    return rawEvents.Records.map((record) => {
        var _a, _b;
        if (!((_a = record.dynamodb) === null || _a === void 0 ? void 0 : _a.NewImage)) {
            throw new Error('Received a DynamoDB stream event without "NewImage" field. It is required');
        }
        return dynamodb_1.Converter.unmarshall((_b = record.dynamodb) === null || _b === void 0 ? void 0 : _b.NewImage);
    });
}
exports.rawEventsToEnvelopes = rawEventsToEnvelopes;
async function readEntityEventsSince(dynamoDB, config, entityTypeName, entityID, since) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'ReadModelStore#readEntityEventsSince');
    const fromTime = since ? since : originOfTime;
    const result = await dynamoDB
        .query({
        TableName: config.resourceNames.eventsStore,
        ConsistentRead: true,
        KeyConditionExpression: `${constants_1.eventsStoreAttributes.partitionKey} = :partitionKey AND ${constants_1.eventsStoreAttributes.sortKey} > :fromTime`,
        ExpressionAttributeValues: {
            ':partitionKey': (0, keys_helper_1.partitionKeyForEvent)(entityTypeName, entityID),
            ':fromTime': fromTime,
        },
        ScanIndexForward: true, // Ascending order (older timestamps first)
    })
        .promise();
    logger.debug(`[EventsAdapter#readEntityEventsSince] Loaded events for entity ${entityTypeName} with ID ${entityID} with result:`, result.Items);
    return result.Items;
}
exports.readEntityEventsSince = readEntityEventsSince;
async function readEntityLatestSnapshot(dynamoDB, config, entityTypeName, entityID) {
    var _a;
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'ReadModelStore#readEntityLatestSnapshot');
    const result = await dynamoDB
        .query({
        TableName: config.resourceNames.eventsStore,
        ConsistentRead: true,
        KeyConditionExpression: `${constants_1.eventsStoreAttributes.partitionKey} = :partitionKey`,
        ExpressionAttributeValues: {
            ':partitionKey': (0, keys_helper_1.partitionKeyForEvent)(entityTypeName, entityID, 'snapshot'),
        },
        ScanIndexForward: false,
        Limit: 1,
    })
        .promise();
    const snapshot = (_a = result.Items) === null || _a === void 0 ? void 0 : _a.shift();
    if (snapshot) {
        logger.debug(`[EventsAdapter#readEntityLatestSnapshot] Snapshot found for entity ${entityTypeName} with ID ${entityID}:`, snapshot);
        return snapshot;
    }
    else {
        logger.debug(`[EventsAdapter#readEntityLatestSnapshot] No snapshot found for entity ${entityTypeName} with ID ${entityID}.`);
        return null;
    }
}
exports.readEntityLatestSnapshot = readEntityLatestSnapshot;
async function storeEvents(dynamoDB, eventEnvelopes, config) {
    const logger = (0, framework_common_helpers_1.getLogger)(config, 'ReadModelStore#storeEvents');
    logger.debug('[EventsAdapter#storeEvents] Storing the following event envelopes:', eventEnvelopes);
    // const putRequests = []
    for (const eventEnvelope of eventEnvelopes) {
        await (0, framework_common_helpers_1.retryIfError)(() => persistEvent(dynamoDB, config, eventEnvelope), framework_types_1.OptimisticConcurrencyUnexpectedVersionError);
    }
    logger.debug('[EventsAdapter#storeEvents] EventEnvelopes stored');
}
exports.storeEvents = storeEvents;
async function persistEvent(dynamoDB, config, eventEnvelope) {
    try {
        const partitionKey = (0, keys_helper_1.partitionKeyForEvent)(eventEnvelope.entityTypeName, eventEnvelope.entityID, eventEnvelope.kind);
        // Generate a new timestamp as sorting key on every try until we can persist the event.
        // This way we guarantee ordering even with events that are stored in the same millisecond
        const sortKey = new Date().toISOString();
        await dynamoDB
            .put({
            TableName: config.resourceNames.eventsStore,
            ConditionExpression: `${constants_1.eventsStoreAttributes.partitionKey} <> :partitionKey AND ${constants_1.eventsStoreAttributes.sortKey} <> :sortKey`,
            ExpressionAttributeValues: {
                ':partitionKey': partitionKey,
                ':sortKey': sortKey,
            },
            Item: {
                ...eventEnvelope,
                [constants_1.eventsStoreAttributes.partitionKey]: partitionKey,
                [constants_1.eventsStoreAttributes.sortKey]: sortKey,
                [constants_1.eventsStoreAttributes.indexByEntity.partitionKey]: (0, keys_helper_1.partitionKeyForIndexByEntity)(eventEnvelope.entityTypeName, eventEnvelope.kind),
            },
        })
            .promise();
    }
    catch (e) {
        const error = e;
        if (error.name == 'ConditionalCheckFailedException') {
            throw new framework_types_1.OptimisticConcurrencyUnexpectedVersionError(error.message);
        }
        throw e;
    }
}
