"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Provider = exports.loadInfrastructurePackage = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = require("aws-sdk");
const api_gateway_io_1 = require("./library/api-gateway-io");
const connections_adapter_1 = require("./library/connections-adapter");
const events_adapter_1 = require("./library/events-adapter");
const events_searcher_adapter_1 = require("./library/events-searcher-adapter");
const graphql_adapter_1 = require("./library/graphql-adapter");
const read_models_adapter_1 = require("./library/read-models-adapter");
const read_models_searcher_adapter_1 = require("./library/read-models-searcher-adapter");
const scheduled_adapter_1 = require("./library/scheduled-adapter");
const subscription_adapter_1 = require("./library/subscription-adapter");
const dynamoDB = new aws_sdk_1.DynamoDB.DocumentClient({
    maxRetries: 10,
    httpOptions: {
        timeout: 2000,
    },
});
/* We load the infrastructure package dynamically here to avoid including it in the
 * dependencies that are deployed in the lambda functions. The infrastructure
 * package is only used during the deploy.
 * Notice that this is done in a separate function to ease testing
 */
function loadInfrastructurePackage(packageName) {
    return require(packageName);
}
exports.loadInfrastructurePackage = loadInfrastructurePackage;
/**
 * `Provider` is a function that accepts a list of rocket names and returns an
 * object compatible with the `ProviderLibrary` defined in the `framework-types` package.
 * The rocket names are passed to the infrastructure package, which loads them dynamically
 * to extend the AWS functionality. Rockets are typically distributed in separate node packages.
 */
const Provider = (rockets) => {
    return {
        // ProviderEventsLibrary
        events: {
            rawToEnvelopes: events_adapter_1.rawEventsToEnvelopes,
            forEntitySince: events_adapter_1.readEntityEventsSince.bind(null, dynamoDB),
            latestEntitySnapshot: events_adapter_1.readEntityLatestSnapshot.bind(null, dynamoDB),
            search: events_searcher_adapter_1.searchEvents.bind(null, dynamoDB),
            store: events_adapter_1.storeEvents.bind(null, dynamoDB),
            searchEntitiesIDs: events_searcher_adapter_1.searchEntitiesIds.bind(null, dynamoDB),
        },
        // ProviderReadModelsLibrary
        readModels: {
            rawToEnvelopes: read_models_adapter_1.rawReadModelEventsToEnvelopes,
            fetch: read_models_adapter_1.fetchReadModel.bind(null, dynamoDB),
            search: read_models_searcher_adapter_1.searchReadModel.bind(null, dynamoDB),
            store: read_models_adapter_1.storeReadModel.bind(null, dynamoDB),
            delete: read_models_adapter_1.deleteReadModel.bind(null, dynamoDB),
            subscribe: subscription_adapter_1.subscribeToReadModel.bind(null, dynamoDB),
            fetchSubscriptions: subscription_adapter_1.fetchSubscriptions.bind(null, dynamoDB),
            deleteSubscription: subscription_adapter_1.deleteSubscription.bind(null, dynamoDB),
            deleteAllSubscriptions: subscription_adapter_1.deleteAllSubscriptions.bind(null, dynamoDB),
        },
        // ProviderGraphQLLibrary
        graphQL: {
            rawToEnvelope: graphql_adapter_1.rawGraphQLRequestToEnvelope,
            handleResult: api_gateway_io_1.requestSucceeded,
        },
        // ProviderAPIHandling
        api: {
            requestSucceeded: api_gateway_io_1.requestSucceeded,
            requestFailed: api_gateway_io_1.requestFailed,
        },
        connections: {
            storeData: connections_adapter_1.storeConnectionData.bind(null, dynamoDB),
            fetchData: connections_adapter_1.fetchConnectionData.bind(null, dynamoDB),
            deleteData: connections_adapter_1.deleteConnectionData.bind(null, dynamoDB),
            sendMessage: connections_adapter_1.sendMessageToConnection,
        },
        // ScheduledCommandsLibrary
        scheduled: {
            rawToEnvelope: scheduled_adapter_1.rawScheduledInputToEnvelope,
        },
        // ProviderInfrastructureGetter
        infrastructure: () => {
            const infrastructurePackageName = require('../package.json').name + '-infrastructure';
            let infrastructure;
            try {
                infrastructure = loadInfrastructurePackage(infrastructurePackageName);
            }
            catch (e) {
                throw new Error(`The AWS infrastructure package could not be loaded. The following error was thrown: ${e.message}. Please ensure that one of the following actions has been done:\n` +
                    `  - It has been specified in your "devDependencies" section of your "package.json" file. You can do so by running 'npm install --save-dev ${infrastructurePackageName}'\n` +
                    `  - Or it has been installed globally. You can do so by running 'npm install -g ${infrastructurePackageName}'`);
            }
            return infrastructure.Infrastructure(rockets);
        },
    };
};
exports.Provider = Provider;
tslib_1.__exportStar(require("./constants"), exports);
